import math
import time
import krpc

turn_start_altitude = 250
turn_end_altitude = 45000
target_altitude = 150000

conn = krpc.connect(name='Lancement en orbite')
vessel = conn.space_center.active_vessel

# Mise en place des ressoursces necessaires
ut = conn.add_stream(getattr, conn.space_center, 'ut')
altitude = conn.add_stream(getattr, vessel.flight(), 'mean_altitude')
apoapsis = conn.add_stream(getattr, vessel.orbit, 'apoapsis_altitude')
stage_2_resources = vessel.resources_in_decouple_stage(stage=2, cumulative=False)
srb_fuel = conn.add_stream(stage_2_resources.amount, 'SolidFuel')

# Dernier preparatif
vessel.control.sas = False
vessel.control.rcs = False
vessel.control.throttle = 1.0

# Compte à rebours...
print('3...')
time.sleep(1)
print('2...')
time.sleep(1)
print('1...')
time.sleep(1)
print('Décollage !')

# Mise à feu du premier etage
vessel.control.activate_next_stage()
vessel.auto_pilot.engage()
vessel.auto_pilot.target_pitch_and_heading(90, 90)

# Boucle d'ascension principale
srbs_separated = False
turn_angle = 0
while True:

    # Maneuvre: Gravity turn
    if altitude() > turn_start_altitude and altitude() < turn_end_altitude:
        frac = ((altitude() - turn_start_altitude) /
                (turn_end_altitude - turn_start_altitude))
        new_turn_angle = frac * 90
        if abs(new_turn_angle - turn_angle) > 0.5:
            turn_angle = new_turn_angle
            vessel.auto_pilot.target_pitch_and_heading(90-turn_angle, 90)

    # Separation des SRB lorsqu'ils sont vide
    if not srbs_separated:
        if srb_fuel() < 65:
            vessel.control.activate_next_stage()
            srbs_separated = True
            print('Séparation des SRB')

    # Décelleration quand  on s'approche de l'apoapse visé
    if apoapsis() > target_altitude*0.9:
        print("En approche de l'apoapse visé")
        break

# Exctinction des moteurs quand l'apoapse visé est attente
vessel.control.throttle = 0.25
while apoapsis() < target_altitude:
    pass
print('Apoapse visé atteinte')
vessel.control.throttle = 0.0

# Pause jusqu'à la sortie de l'atmosphere
print("Sortir de l'atmosphère")
while altitude() < 70500:
    pass

# Planification de la poussé de circularisation (en utilisant l'équation vis-viva)
print('Planification de la poussé de circularisation')
mu = vessel.orbit.body.gravitational_parameter
r = vessel.orbit.apoapsis
a1 = vessel.orbit.semi_major_axis
a2 = r
v1 = math.sqrt(mu*((2./r)-(1./a1)))
v2 = math.sqrt(mu*((2./r)-(1./a2)))
delta_v = v2 - v1
node = vessel.control.add_node(
    ut() + vessel.orbit.time_to_apoapsis, prograde=delta_v)

# Calcule du temps de poussé (en utilisant l'équation de Tsiolkovski)
F = vessel.available_thrust
Isp = vessel.specific_impulse * 9.82
m0 = vessel.mass
m1 = m0 / math.exp(delta_v/Isp)
flow_rate = F / Isp
burn_time = (m0 - m1) / flow_rate

# Orientation du vaisseau
print('Orentation du vaisseau pour la poussé circularisation')
vessel.auto_pilot.reference_frame = node.reference_frame
vessel.auto_pilot.target_direction = (0, 1, 0)
vessel.auto_pilot.wait()

# Pause jusqu'à la maneuvre
print("Pause jusqu'à la poussé circularisation")
burn_ut = ut() + vessel.orbit.time_to_apoapsis - (burn_time/2.)
lead_time = 5
conn.space_center.warp_to(burn_ut - lead_time)

# Execution de la maneuvre
print('Pret pour la maneuvre')
time_to_apoapsis = conn.add_stream(getattr, vessel.orbit, 'time_to_apoapsis')
while time_to_apoapsis() - (burn_time/2.) > 0:
    pass
print('Execution de la maneuvre')
vessel.control.throttle = 1.0
time.sleep(burn_time - 0.1)

vessel.control.throttle = 0.0
node.remove()

print('Mise en orbite reussi')
